@@ -586,12 +586,16 @@ static GdkPixbuf *
 totem_gst_buffer_to_pixbuf (GstBuffer *buffer)
 {
   GdkPixbufLoader *loader;
+  GstMapInfo map_info;
   GdkPixbuf *pixbuf = NULL;
   GError *err = NULL;
 
   loader = gdk_pixbuf_loader_new ();
 
-  if (gdk_pixbuf_loader_write (loader, buffer->data, buffer->size, &err) &&
+  if (!gst_buffer_map (buffer, &map_info, GST_MAP_READ))
+    return NULL;
+
+  if (gdk_pixbuf_loader_write (loader, map_info.data, map_info.size, &err) &&
       gdk_pixbuf_loader_close (loader, &err)) {
     pixbuf = gdk_pixbuf_loader_get_pixbuf (loader);
     if (pixbuf)
@@ -600,69 +604,70 @@ totem_gst_buffer_to_pixbuf (GstBuffer *buffer)
     g_warning ("could not convert tag image to pixbuf: %s", err->message);
     g_error_free (err);
   }
-
+  gst_buffer_unmap (buffer, &map_info);
   g_object_unref (loader);
   return pixbuf;
 }
 
-static const GValue *
+static GstSample *
 totem_gst_tag_list_get_cover_real (GstTagList *tag_list)
 {
-  const GValue *cover_value = NULL;
+  GstSample *cover_sample = NULL;
   guint i;
 
   for (i = 0; ; i++) {
-    const GValue *value;
     GstBuffer *buffer;
+    GstSample *sample;
+    GstCaps *caps;
     GstStructure *caps_struct;
     int type;
 
-    value = gst_tag_list_get_value_index (tag_list,
-					  GST_TAG_IMAGE,
-					  i);
-    if (value == NULL)
+    if (!gst_tag_list_get_sample_index (tag_list, GST_TAG_IMAGE, i, &sample))
       break;
 
-    buffer = gst_value_get_buffer (value);
-
-    caps_struct = gst_caps_get_structure (buffer->caps, 0);
+    buffer = gst_sample_get_buffer (sample);
+    caps = gst_sample_get_caps (sample);
+    caps_struct = gst_caps_get_structure (caps, 0);
     gst_structure_get_enum (caps_struct,
 			    "image-type",
 			    GST_TYPE_TAG_IMAGE_TYPE,
 			    &type);
     if (type == GST_TAG_IMAGE_TYPE_UNDEFINED) {
-      if (cover_value == NULL)
-        cover_value = value;
+      if (cover_sample == NULL) {
+        /* take a ref here since we will continue and unref below */
+        cover_sample = gst_sample_ref (sample);
+      }
     } else if (type == GST_TAG_IMAGE_TYPE_FRONT_COVER) {
-      cover_value = value;
+      cover_sample = sample;
       break;
     }
+    gst_sample_unref (sample);
   }
 
-  return cover_value;
+  return cover_sample;
 }
 
 GdkPixbuf *
 totem_gst_tag_list_get_cover (GstTagList *tag_list)
 {
-  const GValue *cover_value;
+  GstSample *cover_sample;
 
   g_return_val_if_fail (tag_list != NULL, FALSE);
 
-  cover_value = totem_gst_tag_list_get_cover_real (tag_list);
+  cover_sample = totem_gst_tag_list_get_cover_real (tag_list);
   /* Fallback to preview */
-  if (!cover_value) {
-    cover_value = gst_tag_list_get_value_index (tag_list,
-						GST_TAG_PREVIEW_IMAGE,
-						0);
+  if (!cover_sample) {
+    gst_tag_list_get_sample_index (tag_list, GST_TAG_PREVIEW_IMAGE, 0,
+                                   &cover_sample);
   }
 
-  if (cover_value) {
+  if (cover_sample) {
     GstBuffer *buffer;
     GdkPixbuf *pixbuf;
 
-    buffer = gst_value_get_buffer (cover_value);
+    buffer = gst_sample_get_buffer (cover_sample);
     pixbuf = totem_gst_buffer_to_pixbuf (buffer);
+    gst_sample_unref (cover_sample);
     return pixbuf;
   }
 
